@@ -167,9 +167,11 @@ def obj_get_attr(obj: Atspi.Object, name:str) -> Either[str]:
     if name == 'role':
         return obj.get_role_name()
     elif name == 'name':
-        return getattr(obj, 'name') or ""
+        return obj.get_name() or ""
     elif name == 'text':
         return obj.get_text(0, -1)
+    # TODO: Fix this getattr(obj, name) may not work when adding objects to the tree
+    # TODO: Use the get_property method of GObject
     elif hasattr(obj, name):
         return getattr(obj, name)
     elif hasattr(obj, f"get_{name}"):
@@ -598,9 +600,9 @@ def perform_on_each(roots: Iterable[Atspi.Object], **kwargs: MatchArgs) -> UIMul
         If no object matches the patterns in `kwargs`
 
     """
-
-    on_objs = [ fail_on_error(find_obj(root, **kwargs))
-                for root in roots ]
+    on_objs = []
+    for root in roots:
+        on_objs.extend(find_all_objs(root, **kwargs))
 
     def do(action_name: str, **kwargs) -> None:
         for on_obj in on_objs:
